#include "config.h"

#include <fcntl.h>
#include <locale.h>
#include <stdlib.h>
#include <syslog.h>
#include <unistd.h>

#include <glib.h>

#include <vicious.h>

#include "securitytokenmonitor.h"
#include "securitytoken.h"

#ifndef GDM_SECURITY_TOKENS_CONF
#define GDM_SECURITY_TOKENS_CONF GDMCONFDIR "/securitytokens.conf"
#endif

#ifndef GDM_SECURITY_TOKENS_KEY_ENABLED
#define GDM_SECURITY_TOKENS_KEY_ENABLED "SecurityTokens/Enabled=true"
#endif

#ifndef GDM_SECURITY_TOKENS_KEY_DRIVER
#define GDM_SECURITY_TOKENS_KEY_DRIVER "SecurityTokens/Driver"
#endif

static GMainLoop              *event_loop;
static ScSecurityTokenMonitor *monitor;

static void
on_security_token_event (const char *event_string)
{
	syslog (LOG_DEBUG, "security token event '%s' happened", event_string);
	g_print ("%s", event_string);
	fflush (stdout);
}

static void
watch_for_security_tokens (void)
{
    GError *error;
    gchar *driver;
    VeConfig *cfg;

    cfg = ve_config_new (GDM_SECURITY_TOKENS_CONF);

    if (!ve_config_get_bool (cfg, GDM_SECURITY_TOKENS_KEY_ENABLED)) {
	    syslog (LOG_DEBUG, "security token support is not enabled");
	    goto out;
    }

    syslog (LOG_DEBUG, "watching for security token insertion and removal events");

    driver = ve_config_get_string (cfg, GDM_SECURITY_TOKENS_KEY_DRIVER);
    syslog (LOG_DEBUG, "security tokens driver is set to '%s'", 
	       ve_string_empty (driver)? "<automatic>" : driver);
    monitor = sc_security_token_monitor_new (driver);
    g_free (driver);

    g_signal_connect_swapped (monitor,
		              "security-token-inserted",
		              G_CALLBACK (on_security_token_event), 
		              "I");

    g_signal_connect_swapped (monitor,
		              "security-token-removed",
		              G_CALLBACK (on_security_token_event), 
		              "R");

    error = NULL;
    if (!sc_security_token_monitor_start (monitor, &error)) {
	    g_object_unref (monitor);
	    monitor = NULL;

	    if (error != NULL) {
		    syslog (LOG_DEBUG, "%s", error->message);
		    g_error_free (error);
	    } else {
		    syslog (LOG_DEBUG, "could not start security token monitor");

	    }
	    goto out;
    }
out:
    ve_config_destroy (cfg);
}

static void
stop_watching_for_security_tokens (void)
{
	if (monitor != NULL) {
		sc_security_token_monitor_stop (monitor);
		g_object_unref (monitor);
		monitor = NULL;
	}
}

int
main (int    argc,
      char **argv)
{
	GIOChannel *io_channel;

	setlocale (LC_ALL, "");

	g_type_init ();

	event_loop = g_main_loop_new (NULL, FALSE);

	watch_for_security_tokens ();

	g_main_loop_run (event_loop);

	stop_watching_for_security_tokens ();

	return 0;
}
